from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os.path

import charmhelpers.core.hookenv
import yaml

import lib.exceptions
import lib.misc


class MidonetGatewayRequiredConfig(dict):
    def __init__(self):
        super(MidonetGatewayRequiredConfig, self).__init__()

        self['config'] = charmhelpers.core.hookenv.config()

        self.required_options = [
            'openstack-release',
            'midonet-flavor',
            'midonet-release',
            'external-network-name',
            'uplink-type',
        ]
        self.required_options_for_enterprise = [
            'midonet-enterprise-username',
            'midonet-enterprise-password',
        ]
        self.required_options_for_static = [
            'static-linux-bridge-address',
            'static-provider-router-address',
            'static-forwarding-network',
            'static-outgoing-masquerade',
        ]
        self.required_options_for_masquerade = [
            'static-outgoing-interface-name',
        ]
        self.required_options_for_bgp = [
            'bgp-local-as-number',
            'bgp-peer-as-number',
            'bgp-advertised-networks',
            'bgp-topology',
            'port-group-name',
        ]

        config_yaml = os.path.join(
            charmhelpers.core.hookenv.charm_dir(),
            'config.yaml',
        )
        with open(config_yaml) as file_object:
            self.config = yaml.load(file_object).get('options', {})

    def _check_required_options(self):
        try:
            if self['config']['midonet-flavor'] == 'opensource':
                pass

            elif self['config']['midonet-flavor'] == 'enterprise':
                self.required_options.extend(
                    self.required_options_for_enterprise
                )

            else:
                raise lib.exceptions.EnumConfigError

        except KeyError:
            raise lib.exceptions.NotReadyConfigError

        try:
            if self['config']['uplink-type'] == 'static':
                self.required_options.extend(
                    self.required_options_for_static
                )

                if self['config']['static-outgoing-masquerade']:
                    self.required_options.extend(
                        self.required_options_for_masquerade,
                    )

            elif self['config']['uplink-type'] == 'bgp':
                self.required_options.extend(
                    self.required_options_for_bgp
                )

            else:
                raise lib.exceptions.EnumConfigError

        except KeyError:
            raise lib.exceptions.NotReadyConfigError

    def _check_required_options_again(self):
        for required_option in self.required_options:
            try:
                if self['config'][required_option] in (None, ''):
                    raise lib.exceptions.NotReadyConfigError
            except KeyError:
                raise lib.exceptions.NotReadyConfigError

    def __bool__(self):
        try:
            self._check_required_options()
            self._check_required_options_again()
        except lib.exceptions.ConfigError:
            return False

        return True

    def __nonzero__(self):
        return self.__bool__()


def check_config_consistency(rd_config):
    is_leader = charmhelpers.core.hookenv.is_leader()

    if rd_config.changed('external-network-name'):
        raise lib.exceptions.ExternalNetworkNameChangeError

    if rd_config['uplink-type'] == 'static' and not is_leader:
        raise lib.exceptions.MultipleUnitStaticRoutingError

    if rd_config.changed('port-group-name'):
        raise lib.exceptions.PortGroupNameChangeError


def is_config_different(rd_config):
    for key in rd_config:
        if rd_config.changed(key):
            return True

    return False
