from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import subprocess

try:
    import charmhelpers  # noqa pylint: disable=unused-import
except ImportError:
    subprocess.check_call(['apt-get', 'install', '-y', 'python-pip'])
    subprocess.check_call(['pip', 'install', 'charmhelpers'])

import charmhelpers.core.hookenv
import charmhelpers.core.host
import charmhelpers.fetch

import lib.repositories


def _add_midonet_sources(midonet_flavor, midonet_release, openstack_release):
    distrib_codename = charmhelpers.core.host.lsb_release()['DISTRIB_CODENAME']

    get_sources_functions = {
        'trusty': lib.repositories.get_trusty_sources,
    }
    apt_sources = get_sources_functions[distrib_codename](
        midonet_flavor,
        midonet_release,
        openstack_release,
    )

    for apt_source in apt_sources:
        charmhelpers.fetch.add_source(
            apt_source.repository,
            apt_source.key,
        )


def _add_openstack_sources(openstack_release):
    supported_openstack_releases = {
        'kilo': 'kilo',
        'liberty': 'liberty',
    }
    charmhelpers.fetch.add_source(
        'cloud-archive:{}'.format(
            supported_openstack_releases[openstack_release],
        ),
    )


def main():
    charmhelpers.core.hookenv.log(
        'Install API client packages if necessary.',
        charmhelpers.core.hookenv.INFO,
    )

    config = charmhelpers.core.hookenv.config()

    _add_midonet_sources(
        midonet_flavor=config['midonet-flavor'],
        midonet_release=config['midonet-release'],
        openstack_release=config['openstack-release'],
    )
    _add_openstack_sources(
        openstack_release=config['openstack-release'],
    )
    charmhelpers.fetch.apt_update()

    required_packages = [
        'libaugeas0',
        'patch',
        'python-keystoneclient',
        'python-midonetclient',
        'python-netaddr',
        'python-netifaces',
        'python-neutronclient',
        'python-requests',
        'python-retrying',
        'python-webob',
        'python-yaml',
    ]

    missing_packages = charmhelpers.fetch.filter_installed_packages(
        required_packages,
    )

    if missing_packages:
        charmhelpers.core.hookenv.log(
            'Installing {}.'.format(', '.join(missing_packages)),
            charmhelpers.core.hookenv.INFO,
        )
        charmhelpers.fetch.apt_install(missing_packages)

    patchfile_path = os.path.abspath(os.path.join(
        os.path.dirname(__file__),
        os.path.pardir,
        'files/python-midonetclient_v2016.06.3.patch',
    ))
    patch_dry_run_result = subprocess.call(
        args=['patch', '-i', patchfile_path, '-p0', '-N', '--dry-run'],
        cwd='/usr/lib/python2.7/dist-packages/midonetclient',
    )
    if patch_dry_run_result == 0:
        subprocess.check_call(
            args=['patch', '-i', patchfile_path, '-p0'],
            cwd='/usr/lib/python2.7/dist-packages/midonetclient',
        )
