from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import lib.log
import lib.misc
import lib.workarounds


@lib.misc.retry_neutron_exception
def get_external_network(neutron_client, external_network_name):
    external_network = None

    for network in neutron_client.list_networks()['networks']:
        if external_network_name == network['name']:
            external_network = network

    if external_network:
        lib.log.debug(
            lib.log.NEUTRON_EXTERNAL_NETWORK_FOUND.format(
                external_network_name,
                external_network['id'],
            ),
        )
    else:
        lib.log.debug(
            lib.log.NEUTRON_EXTERNAL_NETWORK_NOT_FOUND.format(
                external_network_name,
            ),
        )

    return external_network


@lib.misc.retry_neutron_exception_result
def get_existing_external_network(neutron_client, external_network_name):
    external_network = None

    for network in neutron_client.list_networks()['networks']:
        if external_network_name == network['name']:
            external_network = network

    if external_network:
        lib.log.debug(
            lib.log.NEUTRON_EXTERNAL_NETWORK_FOUND.format(
                external_network_name,
                external_network['id'],
            ),
        )
    else:
        lib.log.debug(
            lib.log.NEUTRON_EXTERNAL_NETWORK_NOT_FOUND.format(
                external_network_name,
            ),
        )

    return external_network


@lib.misc.retry_neutron_exception
def create_external_network(neutron_client, external_network_name):
    external_network = get_external_network(
        neutron_client,
        external_network_name,
    )

    if not external_network:
        lib.workarounds.wait_for_midonet_api_to_settle()

        external_network = neutron_client.create_network({
            'network': {
                'name': external_network_name,
                'router:external': True,
            }
        })['network']
        lib.log.debug(
            lib.log.NEUTRON_EXTERNAL_NETWORK_CREATED.format(
                external_network_name,
                external_network['id'],
            ),
        )

    return external_network
