from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from charmhelpers.core.host import service_running
from charmhelpers.core.services.helpers import RelationContext

import lib.log


class IdentityAdminRelation(RelationContext):
    name = 'identity-admin'
    interface = 'keystone-admin'
    required_keys = [
        'service_password',
        'service_tenant_name',
        'service_username',
    ]

    def is_ready(self):
        ready = len(self.get(self.name, [])) > 0
        if not ready:
            lib.log.info_with_status(
                'waiting',
                'Incomplete relation: {}'.format(self.__class__.__name__),
            )
        return ready


class NeutronPluginApiRelation(RelationContext):
    name = 'neutron-plugin-api'
    interface = 'neutron-plugin-api'
    required_keys = [
        'service_host',
        'service_port',
        'service_protocol',
    ]

    def is_ready(self):
        ready = len(self.get(self.name, [])) > 0
        if not ready:
            lib.log.info_with_status(
                'waiting',
                'Incomplete relation: {}'.format(self.__class__.__name__),
            )
        return ready


class MidonetApiRelation(RelationContext):
    name = 'midonet-api'
    interface = 'midonet'
    required_keys = [
        'host',
        'port',
    ]

    def is_ready(self):
        ready = len(self.get(self.name, [])) > 0
        if not ready:
            lib.log.info_with_status(
                'waiting',
                'Incomplete relation: {}'.format(self.__class__.__name__),
            )
        return ready


class MidonetHostRelation(RelationContext):
    name = 'midonet-host'
    interface = 'midonet-host'

    def is_ready(self):
        data_ready = len(self.get(self.name, [])) > 0
        service_ready = service_running('midolman')

        ready = data_ready and service_ready

        if not ready:
            lib.log.info_with_status(
                'waiting',
                'Incomplete relation: {}'.format(self.__class__.__name__),
            )
        return ready
