from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import collections
import os.path

import charmhelpers.core.hookenv

AptSource = collections.namedtuple('AptSource', ['repository', 'key'])


def _read_apt_key(file_name):
    file_path = os.path.join(
        charmhelpers.core.hookenv.charm_dir(), 'files', 'apt-keys', file_name,
    )
    with open(file_path, 'r') as file_object:
        return file_object.read()


def _repo_enterprise_midonet_19():
    config = charmhelpers.core.hookenv.config()
    repository_url = (
        'deb http://{}:{}@apt.midokura.com/midonet/v1.9/stable trusty main'
    )
    return AptSource(
        repository=repository_url.format(
            config['midonet-enterprise-username'],
            config['midonet-enterprise-password'],
        ),
        key=_read_apt_key('apt_midokura_com.key'),
    )


def _repo_enterprise_midonet_50():
    config = charmhelpers.core.hookenv.config()
    repository_url = (
        'deb http://{}:{}@repo.midokura.com/mem-5 stable main'
    )
    return AptSource(
        repository=repository_url.format(
            config['midonet-enterprise-username'],
            config['midonet-enterprise-password'],
        ),
        key=_read_apt_key('common.key'),
    )


def _repo_opensource_midonet_201506():
    repository_url = (
        'deb http://repo.midonet.org/midonet/v2015.06 stable main'
    )
    return AptSource(
        repository=repository_url,
        key=_read_apt_key('repo_midonet_org.key'),
    )


def _repo_opensource_midonet_50():
    repository_url = (
        'deb http://builds.midonet.org/midonet-5 stable main'
    )
    return AptSource(
        repository=repository_url,
        key=_read_apt_key('repo_midonet_org.key'),
    )


def _repo_openstack_plugins_kilo():
    repository_url = (
        'deb http://builds.midonet.org/openstack-kilo stable main'
    )
    return AptSource(
        repository=repository_url,
        key=_read_apt_key('common.key'),
    )


def _repo_openstack_plugins_liberty():
    repository_url = (
        'deb http://builds.midonet.org/openstack-liberty stable main'
    )
    return AptSource(
        repository=repository_url,
        key=_read_apt_key('common.key'),
    )


def _repo_opensource_midonet_misc():
    repository_url = (
        'deb http://builds.midonet.org/misc stable main'
    )
    return AptSource(
        repository=repository_url,
        key=_read_apt_key('common.key'),
    )


def _get_trusty_midonet_ent_sources(midonet_release, openstack_release):
    apt_sources = []

    if midonet_release == '1.9':
        apt_sources.append(_repo_enterprise_midonet_19())

        plugins_functions = {
            'kilo': _repo_openstack_plugins_kilo,
            'liberty': _repo_openstack_plugins_liberty,
        }
        apt_sources.append(plugins_functions[openstack_release]())

    elif midonet_release == '5.0':
        apt_sources.append(_repo_enterprise_midonet_50())

        plugins_functions = {
            'kilo': _repo_openstack_plugins_kilo,
            'liberty': _repo_openstack_plugins_liberty,
        }
        apt_sources.append(plugins_functions[openstack_release]())

    else:
        raise KeyError

    return apt_sources


def _get_trusty_midonet_oss_sources(midonet_release, openstack_release):
    apt_sources = []

    if midonet_release == '2015.06':
        apt_sources.append(_repo_opensource_midonet_201506())

        plugins_functions = {
            'kilo': _repo_openstack_plugins_kilo,
        }
        apt_sources.append(plugins_functions[openstack_release]())

    elif midonet_release == '5.0':
        apt_sources.append(_repo_opensource_midonet_50())

        plugins_functions = {
            'kilo': _repo_openstack_plugins_kilo,
            'liberty': _repo_openstack_plugins_liberty,
        }
        apt_sources.append(plugins_functions[openstack_release]())

    else:
        raise KeyError

    apt_sources.append(_repo_opensource_midonet_misc())

    return apt_sources


def get_trusty_sources(midonet_flavor, midonet_release, openstack_release):
    get_midonet_sources_functions = {
        'enterprise': _get_trusty_midonet_ent_sources,
        'opensource': _get_trusty_midonet_oss_sources,
    }
    apt_sources = get_midonet_sources_functions[midonet_flavor](
        midonet_release,
        openstack_release,
    )

    return apt_sources
