from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestHost(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_host_list()

        gateway = lib.midonet_api.get_gateway_node(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
            host_uuid=lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        )

        assert gateway.get_id() == str(
            lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        )
        assert gateway.get_name() == str(
            lib.tests.mock_midonet_api.HOSTNAME_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_HOST_FOUND.format(
                lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                lib.tests.mock_midonet_api.HOSTNAME_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )
