from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestHostPort(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()
        lib.tests.mock_midonet_api.mock_routers_list_pass()
        lib.tests.mock_midonet_api.mock_host_list()
        lib.tests.mock_midonet_api.mock_router_port_list_pass()

        self.router = lib.midonet_api.get_provider_router(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
        )
        self.host = lib.midonet_api.get_gateway_node(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
            host_uuid=lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        )
        self.port = lib.midonet_api.router_get_port(
            router=self.router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_host_port_list_pass()

        host_interface_port = lib.midonet_api.host_get_binding(
            host=self.host,
            port=self.port,
            interface=lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
        )

        assert host_interface_port.get_host_id() == str(
            lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        )
        assert host_interface_port.get_interface_name() == str(
            lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
        )
        assert host_interface_port.get_port_id() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_HOST_INTERFACE_PORT_FOUND.format(
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_list_fail(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_host_port_list_fail()

        host_interface_port = lib.midonet_api.host_get_binding(
            host=self.host,
            port=self.port,
            interface=lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
        )

        assert host_interface_port is None
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_HOST_INTERFACE_PORT_NOT_FOUND.format(
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_add(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_host_port_list_fail()
        lib.tests.mock_midonet_api.mock_host_port_add()
        lib.tests.mock_midonet_api.mock_host_port_get()

        host_interface_port = lib.midonet_api.host_add_binding(
            host=self.host,
            port=self.port,
            interface=lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
        )

        assert host_interface_port.get_host_id() == str(
            lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        )
        assert host_interface_port.get_interface_name() == str(
            lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
        )
        assert host_interface_port.get_port_id() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS
        )
        self.mocked_log.assert_called_with(
            lib.log.MIDONET_HOST_INTERFACE_PORT_CREATED.format(
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mock_midonet_api.INTERFACE_NAME_PASS,
                lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )
