from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestPortBGP(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()
        lib.tests.mock_midonet_api.mock_routers_list_pass()
        lib.tests.mock_midonet_api.mock_router_port_list_pass()

        self.provider_router = lib.midonet_api.get_provider_router(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
        )
        self.port = lib.midonet_api.router_get_port(
            router=self.provider_router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_bgp_list_pass()

        bgp = lib.midonet_api.port_get_bgp(
            port=self.port,
            local_as=lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
            peer_address=str(
                lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS
            ),
            peer_as=lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
        )

        assert bgp.get_local_as() == int(
            lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
        )
        assert bgp.get_peer_addr() == str(
            lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS,
        )
        assert bgp.get_peer_as() == int(
            lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
        )
        assert bgp.get_id() == str(
            lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PORT_BGP_FOUND.format(
                lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
                lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS,
                lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
                lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_list_fail(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_bgp_list_fail()

        bgp = lib.midonet_api.port_get_bgp(
            port=self.port,
            local_as=lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
            peer_address=str(
                lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS
            ),
            peer_as=lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
        )

        assert bgp is None
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PORT_BGP_NOT_FOUND.format(
                lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
                lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS,
                lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_add(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_bgp_list_fail()
        lib.tests.mock_midonet_api.mock_port_bgp_add()
        lib.tests.mock_midonet_api.mock_bgp_get()

        bgp = lib.midonet_api.port_add_bgp(
            port=self.port,
            local_as=lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
            peer_address=str(
                lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS
            ),
            peer_as=lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
        )
        assert bgp.get_local_as() == int(
            lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
        )
        assert bgp.get_peer_addr() == str(
            lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS,
        )
        assert bgp.get_peer_as() == int(
            lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
        )
        assert bgp.get_id() == str(
            lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
        )
        self.mocked_log.assert_called_with(
            lib.log.MIDONET_PORT_BGP_CREATED.format(
                lib.tests.mock_midonet_api.AS_NUMBER_LOCAL_PASS,
                lib.tests.mock_midonet_api.IPADDRESS_PEER_PORT_PASS,
                lib.tests.mock_midonet_api.AS_NUMBER_PEER_PASS,
                lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )
