from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestPortGroup(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_group_list_pass()

        port_group = lib.midonet_api.get_stateful_port_group(
            client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
            name=lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
        )

        assert port_group.get_id() == str(
            lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        )
        assert port_group.get_name() == str(
            lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PORT_GROUP_FOUND.format(
                lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
                lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_list_fail(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_group_list_fail()

        port_group = lib.midonet_api.get_stateful_port_group(
            client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
            name=lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
        )

        assert port_group is None
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PORT_GROUP_NOT_FOUND.format(
                lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
                lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_add(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_group_list_fail()
        lib.tests.mock_midonet_api.mock_port_group_add()
        lib.tests.mock_midonet_api.mock_port_group_get()

        port_group = lib.midonet_api.create_stateful_port_group(
            client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
            name=lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
            tenant_id=lib.tests.mock_midonet_api.PORT_GROUP_TENANT_ID_PASS,
        )

        assert port_group.get_id() == str(
            lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        )
        assert port_group.get_name() == str(
            lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
        )
        self.mocked_log.assert_called_with(
            lib.log.MIDONET_PORT_GROUP_CREATED.format(
                lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
                lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )
