from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestPortGroupPort(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()
        lib.tests.mock_midonet_api.mock_routers_list_pass()
        lib.tests.mock_midonet_api.mock_router_port_list_pass()
        lib.tests.mock_midonet_api.mock_port_group_list_pass()

        self.router = lib.midonet_api.get_provider_router(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
        )
        self.port = lib.midonet_api.router_get_port(
            router=self.router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )
        self.port_group = lib.midonet_api.get_stateful_port_group(
            client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
            name=lib.tests.mock_midonet_api.PORT_GROUP_NAME_PASS,
        )

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_group_port_list_pass()

        port_group_port = lib.midonet_api.port_group_get_port(
            port_group=self.port_group,
            port=self.port,
        )

        assert port_group_port.get_port_group_id() == str(
            lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        )
        assert port_group_port.get_port_id() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PORT_GROUP_PORT_FOUND.format(
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_list_fail(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_group_port_list_fail()

        port_group_port = lib.midonet_api.port_group_get_port(
            port_group=self.port_group,
            port=self.port,
        )

        assert port_group_port is None
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PORT_GROUP_PORT_NOT_FOUND.format(
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_add(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_port_group_port_list_fail()
        lib.tests.mock_midonet_api.mock_port_group_port_add()
        lib.tests.mock_midonet_api.mock_port_group_port_get()

        port_group_port = lib.midonet_api.port_group_add_port(
            port_group=self.port_group,
            port=self.port,
        )

        assert port_group_port.get_port_group_id() == str(
            lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        )
        assert port_group_port.get_port_id() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        )
        self.mocked_log.assert_called_with(
            lib.log.MIDONET_PORT_GROUP_PORT_CREATED.format(
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )
