from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestRouters(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_routers_list_pass()

        provider_router = lib.midonet_api.get_provider_router(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
        )

        assert provider_router.get_name() == 'MidoNet Provider Router'
        assert provider_router.get_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_PROVIDER_ROUTER_FOUND.format(
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )
