from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import netaddr

import lib.tests.mocks.midonet


IPNETWORK_PORT_PASS = netaddr.IPNetwork('192.0.2.9/30')
IPNETWORK_PORT_FAIL = netaddr.IPNetwork('192.0.2.13/30')
AS_NUMBER_LOCAL_PASS = 64512
AS_NUMBER_LOCAL_FAIL = 64514
IPADDRESS_PEER_PORT_PASS = netaddr.IPAddress('192.0.2.10')
IPADDRESS_PEER_PORT_FAIL = netaddr.IPAddress('192.0.2.14')
AS_NUMBER_PEER_PASS = 64513
AS_NUMBER_PEER_FAIL = 64515
IPNETWORK_SRC_NET_PASS = netaddr.IPNetwork('198.51.100.64/28')
IPNETWORK_SRC_NET_FAIL = netaddr.IPNetwork('198.51.100.80/28')
IPNETWORK_DST_NET_PASS = netaddr.IPNetwork('198.51.100.96/28')
IPNETWORK_DST_NET_FAIL = netaddr.IPNetwork('198.51.100.112/28')
HOSTNAME_PASS = 'midonet-gateway-1'
TZONE_NAME = 'default_tz'
TZONE_TYPE = 'gre'
IPADDRESS_HOST_PASS = netaddr.IPAddress('203.0.113.2')
INTERFACE_NAME_PASS = 'veth2'
INTERFACE_NAME_FAIL = 'veth3'
PORT_GROUP_NAME_PASS = 'portGroupPass'
PORT_GROUP_NAME_FAIL = 'portGroupFail'
PORT_GROUP_TENANT_ID_PASS = 'c680e63d-a56c-40a5-8976-ef10aa1d5b66'


def mock_login():
    lib.tests.mocks.midonet.mock_login(
        lib.tests.mocks.midonet.UUID_AUTH_TOKEN,
    )


def mock_root():
    lib.tests.mocks.midonet.mock_root()


def mock_system_state():
    lib.tests.mocks.midonet.mock_system_state()


def mock_routers_list_pass():
    lib.tests.mocks.midonet.mock_routers_list(
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        'MidoNet Provider Router',
    )


def mock_router_port_list_pass():
    lib.tests.mocks.midonet.mock_router_port_list(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        IPNETWORK_PORT_PASS,
    )


def mock_router_port_list_fail():
    lib.tests.mocks.midonet.mock_router_port_list(
        lib.tests.mocks.midonet.UUID_PORT_ID_FAIL,
        lib.tests.mocks.midonet.UUID_HOST_ID_FAIL,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        IPNETWORK_PORT_FAIL,
    )


def mock_router_port_add():
    lib.tests.mocks.midonet.mock_router_port_add(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
    )


def mock_port_delete():
    lib.tests.mocks.midonet.mock_port_delete(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_port_get():
    lib.tests.mocks.midonet.mock_port_get(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        IPNETWORK_PORT_PASS,
    )


def mock_port_bgp_list_pass():
    lib.tests.mocks.midonet.mock_port_bgp_list(
        lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        AS_NUMBER_LOCAL_PASS,
        IPADDRESS_PEER_PORT_PASS,
        AS_NUMBER_PEER_PASS,
    )


def mock_port_bgp_list_fail():
    lib.tests.mocks.midonet.mock_port_bgp_list(
        lib.tests.mocks.midonet.UUID_BGP_ID_FAIL,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        AS_NUMBER_LOCAL_FAIL,
        IPADDRESS_PEER_PORT_FAIL,
        AS_NUMBER_PEER_FAIL,
    )


def mock_port_bgp_add():
    lib.tests.mocks.midonet.mock_port_bgp_add(
        lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_bgp_get():
    lib.tests.mocks.midonet.mock_bgp_get(
        lib.tests.mocks.midonet.UUID_BGP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        AS_NUMBER_LOCAL_PASS,
        IPADDRESS_PEER_PORT_PASS,
        AS_NUMBER_PEER_PASS,
    )


def mock_routers_route_list_pass():
    lib.tests.mocks.midonet.mock_routers_route_list(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        IPNETWORK_SRC_NET_PASS,
        IPNETWORK_DST_NET_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_routers_route_list_fail():
    lib.tests.mocks.midonet.mock_routers_route_list(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_FAIL,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        IPNETWORK_SRC_NET_FAIL,
        IPNETWORK_DST_NET_FAIL,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_routers_route_add():
    lib.tests.mocks.midonet.mock_routers_route_add(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
    )


def mock_route_delete():
    lib.tests.mocks.midonet.mock_route_delete(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
    )


def mock_route_get():
    lib.tests.mocks.midonet.mock_route_get(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        IPNETWORK_SRC_NET_PASS,
        IPNETWORK_DST_NET_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_host_list():
    lib.tests.mocks.midonet.mock_host_list(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        HOSTNAME_PASS,
    )


def mock_tunnel_zone_list():
    lib.tests.mocks.midonet.mock_tunnel_zone_list(
        lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
        TZONE_NAME,
        TZONE_TYPE,
    )


def mock_tunnel_zone_get():
    lib.tests.mocks.midonet.mock_tunnel_zone_get(
        lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
        TZONE_NAME,
        TZONE_TYPE,
    )


def mock_tunnel_zone_host_list():
    lib.tests.mocks.midonet.mock_tunnel_zone_host_list(
        lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        IPADDRESS_HOST_PASS,
    )


def mock_host_port_list_pass():
    lib.tests.mocks.midonet.mock_host_port_list(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        INTERFACE_NAME_PASS,
    )


def mock_host_port_list_fail():
    lib.tests.mocks.midonet.mock_host_port_list(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_FAIL,
        INTERFACE_NAME_FAIL,
    )


def mock_host_port_add():
    lib.tests.mocks.midonet.mock_host_port_add(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_host_port_get():
    lib.tests.mocks.midonet.mock_host_port_get(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        INTERFACE_NAME_PASS,
    )


def mock_host_interface_port_delete():
    lib.tests.mocks.midonet.mock_host_interface_port_delete(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_port_group_list_pass():
    lib.tests.mocks.midonet.mock_port_group_list(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        PORT_GROUP_NAME_PASS,
        PORT_GROUP_TENANT_ID_PASS,
    )


def mock_port_group_list_fail():
    lib.tests.mocks.midonet.mock_port_group_list(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_FAIL,
        PORT_GROUP_NAME_FAIL,
        PORT_GROUP_TENANT_ID_PASS,
    )


def mock_port_group_add():
    lib.tests.mocks.midonet.mock_port_group_add(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
    )


def mock_port_group_get():
    lib.tests.mocks.midonet.mock_port_group_get(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        PORT_GROUP_NAME_PASS,
        PORT_GROUP_TENANT_ID_PASS,
    )


def mock_port_group_delete():
    lib.tests.mocks.midonet.mock_port_group_delete(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
    )


def mock_port_group_port_list_pass():
    lib.tests.mocks.midonet.mock_port_group_port_list(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_port_group_port_list_fail():
    lib.tests.mocks.midonet.mock_port_group_port_list(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_FAIL,
    )


def mock_port_group_port_list_empty():
    lib.tests.mocks.midonet.mock_port_group_port_list_empty(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
    )


def mock_port_group_port_add():
    lib.tests.mocks.midonet.mock_port_group_port_add(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_port_group_port_get():
    lib.tests.mocks.midonet.mock_port_group_port_get(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_port_group_port_delete():
    lib.tests.mocks.midonet.mock_port_group_port_delete(
        lib.tests.mocks.midonet.UUID_PGROUP_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )
