from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import netaddr

import lib.misc
import lib.tests.mocks.midonet


SOME_HOST_NAME = 'some_hostname'
TZONE_NAME = 'default_tz'
TZONE_TYPE = 'gre'
IPADDRESS_HOST_PASS = netaddr.IPAddress('203.0.113.2')

STATIC_LINUX_BRIDGE_ADDRESS = '192.0.2.1/30'
STATIC_PROVIDER_ROUTER_ADDRESS = '192.0.2.2/30'
STATIC_FORWARDING_NETWORK = '10.20.30.0/24'


def mock_login():
    lib.tests.mocks.midonet.mock_login(
        lib.tests.mocks.midonet.UUID_AUTH_TOKEN,
    )


def mock_root():
    lib.tests.mocks.midonet.mock_root()


def mock_system_state():
    lib.tests.mocks.midonet.mock_system_state()


def mock_routers_list_pass():
    lib.tests.mocks.midonet.mock_routers_list(
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        'MidoNet Provider Router',
    )


def mock_router_port_list_pass():
    lib.tests.mocks.midonet.mock_router_port_list(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        netaddr.IPNetwork(STATIC_PROVIDER_ROUTER_ADDRESS),
    )


def mock_router_port_list_fail():
    lib.tests.mocks.midonet.mock_router_port_list(
        lib.tests.mocks.midonet.UUID_PORT_ID_FAIL,
        lib.tests.mocks.midonet.UUID_HOST_ID_FAIL,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        netaddr.IPNetwork('203.0.113.1/24'),
    )


def mock_router_port_add():
    lib.tests.mocks.midonet.mock_router_port_add(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
    )


def mock_port_get():
    lib.tests.mocks.midonet.mock_port_get(
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        netaddr.IPNetwork(STATIC_PROVIDER_ROUTER_ADDRESS),
    )


def mock_routers_route_list_pass():
    lib.tests.mocks.midonet.mock_routers_route_list(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        netaddr.IPNetwork('0.0.0.0/0'),
        netaddr.IPNetwork('0.0.0.0/0'),
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        netaddr.IPNetwork(STATIC_LINUX_BRIDGE_ADDRESS).ip,
    )


def mock_routers_route_list_fail():
    lib.tests.mocks.midonet.mock_routers_route_list(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_FAIL,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        netaddr.IPNetwork('203.0.113.0/30'),
        netaddr.IPNetwork('203.0.113.4/30'),
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        netaddr.IPNetwork(STATIC_LINUX_BRIDGE_ADDRESS).ip,
    )


def mock_routers_route_add():
    lib.tests.mocks.midonet.mock_routers_route_add(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
    )


def mock_route_get():
    lib.tests.mocks.midonet.mock_route_get(
        lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        netaddr.IPNetwork('0.0.0.0/0'),
        netaddr.IPNetwork('0.0.0.0/0'),
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        netaddr.IPNetwork(STATIC_LINUX_BRIDGE_ADDRESS).ip,
    )


def mock_host_list():
    lib.tests.mocks.midonet.mock_host_list(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        SOME_HOST_NAME,
    )


def mock_tunnel_zone_list():
    lib.tests.mocks.midonet.mock_tunnel_zone_list(
        lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
        TZONE_NAME,
        TZONE_TYPE,
    )


def mock_tunnel_zone_get():
    lib.tests.mocks.midonet.mock_tunnel_zone_get(
        lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
        TZONE_NAME,
        TZONE_TYPE,
    )


def mock_tunnel_zone_host_list():
    lib.tests.mocks.midonet.mock_tunnel_zone_host_list(
        lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        IPADDRESS_HOST_PASS,
    )


def mock_host_port_list_pass():
    lib.tests.mocks.midonet.mock_host_port_list(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
    )


def mock_host_port_list_fail():
    lib.tests.mocks.midonet.mock_host_port_list(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_FAIL,
        'veth99',
    )


def mock_host_port_add():
    lib.tests.mocks.midonet.mock_host_port_add(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
    )


def mock_host_port_get():
    lib.tests.mocks.midonet.mock_host_port_get(
        lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
        lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
    )
