from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import uuid

import httpretty

import lib.misc


UUID_KEYSTONE_ROLE_MEMBER = uuid.uuid4()
UUID_KEYSTONE_ROLE_ADMIN = uuid.uuid4()
UUID_KEYSTONE_ENDPOINT_NEUTRON = uuid.uuid4()
UUID_KEYSTONE_TOKEN = uuid.uuid4()
UUID_KEYSTONE_TENANT_ADMIN = uuid.uuid4()
UUID_KEYSTONE_USER_ADMIN = uuid.uuid4()
UUID_NEUTRON_NET_EXTERNAL_PASS = uuid.uuid4()
UUID_NEUTRON_NET_EXTERNAL_FAIL = uuid.uuid4()

EXTERNAL_NETWORK_NAME_PASS = 'external-network-1'
EXTERNAL_NETWORK_NAME_FAIL = 'external-network-2'

CT_APPLICATION_JSON = 'application/json'


def _body_keystone_tokens():
    return {
        'access': {
            'metadata': {
                'is_admin': 0,
                'roles': [
                    str(UUID_KEYSTONE_ROLE_MEMBER),
                    str(UUID_KEYSTONE_ROLE_ADMIN),
                ],
            },
            'serviceCatalog': [
                {
                    'endpoints': [
                        {
                            'adminURL': 'http://127.0.0.1:9696',
                            'id': str(UUID_KEYSTONE_ENDPOINT_NEUTRON),
                            'internalURL': 'http://127.0.0.1:9696',
                            'publicURL': 'http://127.0.0.1:9696',
                            'region': 'regionOne',
                        },
                    ],
                    'endpoints_links': [],
                    'name': 'neutron',
                    'type': 'network',
                },
            ],
            'token': {
                'audit_ids': [
                    '0000000000000000000000',
                ],
                'expires': '2038-01-19T03:14:07Z',
                'id': str(UUID_KEYSTONE_TOKEN),
                'issued_at': '1970-01-01T00:00:00.000000',
                'tenant': {
                    'description': 'Admin Tenant',
                    'enabled': True,
                    'id': str(UUID_KEYSTONE_TENANT_ADMIN),
                    'name': 'admin',
                },
            },
            'user': {
                'id': str(UUID_KEYSTONE_USER_ADMIN),
                'name': 'admin',
                'roles': [
                    {
                        'name': '_member_',
                    },
                    {
                        'name': 'admin',
                    }
                ],
                'roles_links': [],
                'username': 'admin',
            },
        },
    }


def _body_net_list_pass():
    return {
        'networks': [
            {
                'admin_state_up': True,
                'id': str(UUID_NEUTRON_NET_EXTERNAL_PASS),
                'name': EXTERNAL_NETWORK_NAME_PASS,
                'router:external': True,
                'shared': True,
                'status': 'ACTIVE',
                'subnets': [],
                'tenant_id': str(UUID_KEYSTONE_TENANT_ADMIN),
            },
        ]
    }


def _body_net_list_fail():
    return {
        'networks': [
            {
                'admin_state_up': True,
                'id': str(UUID_NEUTRON_NET_EXTERNAL_FAIL),
                'name': EXTERNAL_NETWORK_NAME_FAIL,
                'router:external': True,
                'shared': True,
                'status': 'ACTIVE',
                'subnets': [],
                'tenant_id': str(UUID_KEYSTONE_TENANT_ADMIN),
            },
        ]
    }


def _body_net_create():
    return {
        'network': {
            'admin_state_up': True,
            'id': str(UUID_NEUTRON_NET_EXTERNAL_PASS),
            'name': EXTERNAL_NETWORK_NAME_PASS,
            'router:external': True,
            'shared': True,
            'status': 'ACTIVE',
            'subnets': [],
            'tenant_id': str(UUID_KEYSTONE_TENANT_ADMIN),
        }
    }


def get_bogus_neutronclient():
    return lib.misc.get_neutronclient(
        {
            'service_username': 'username',
            'service_password': 'password',
            'service_tenant_name': 'tenant_name',
            'service_hostname': '127.0.0.1',
            'service_port': 5000,
        }
    )


def mock_keystone_tokens():
    httpretty.register_uri(
        uri='http://127.0.0.1:5000/v2.0/tokens',
        method=httpretty.POST,
        status=200,
        content_type=CT_APPLICATION_JSON,
        body=json.dumps(_body_keystone_tokens()),
    )


def mock_net_list_pass():
    httpretty.register_uri(
        uri='http://127.0.0.1:9696/v2.0/networks.json',
        method=httpretty.GET,
        status=200,
        content_type=CT_APPLICATION_JSON,
        body=json.dumps(_body_net_list_pass()),
    )


def mock_net_list_fail():
    httpretty.register_uri(
        uri='http://127.0.0.1:9696/v2.0/networks.json',
        method=httpretty.GET,
        status=200,
        content_type=CT_APPLICATION_JSON,
        body=json.dumps(_body_net_list_fail()),
    )


def mock_net_create():
    httpretty.register_uri(
        uri='http://127.0.0.1:9696/v2.0/networks.json',
        method=httpretty.POST,
        status=201,
        content_type=CT_APPLICATION_JSON,
        body=json.dumps(_body_net_create()),
    )
