from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import uuid

import httpretty

import lib.misc


UUID_AUTH_TOKEN = uuid.uuid4()
UUID_BGP_ID_PASS = uuid.uuid4()
UUID_BGP_ID_FAIL = uuid.uuid4()
UUID_HOST_ID_PASS = uuid.uuid4()
UUID_HOST_ID_FAIL = uuid.uuid4()
UUID_PORT_ID_PASS = uuid.uuid4()
UUID_PORT_ID_FAIL = uuid.uuid4()
UUID_ROUTER_ID_PASS = uuid.uuid4()
UUID_ROUTE_ID_PASS = uuid.uuid4()
UUID_ROUTE_ID_FAIL = uuid.uuid4()
UUID_TZONE_ID_PASS = uuid.uuid4()
UUID_PGROUP_ID_PASS = uuid.uuid4()
UUID_PGROUP_ID_FAIL = uuid.uuid4()


API_HOST = '127.0.0.1'
API_PORT = '8080'


CT_TOKEN_V1 = (
    'application/vnd.org.midonet.Token-v1+json;charset=UTF-8',
)
CT_APPLICATION_V5 = (
    'application/vnd.org.midonet.Application-v5+json',
)
CT_SYSTEM_STATE_V2 = (
    'application/vnd.org.midonet.SystemState-v2+json',
)
CT_COLLECTION_ROUTER_V2 = (
    'application/vnd.org.midonet.collection.Router-v2+json',
)
CT_COLLECTION_PORT_V2 = (
    'application/vnd.org.midonet.collection.Port-v2+json',
)
CT_COLLECTION_BGP_V1 = (
    'application/vnd.org.midonet.collection.Bgp-v1+json',
)
CT_COLLECTION_ROUTE_V1 = (
    'application/vnd.org.midonet.collection.Route-v1+json',
)
CT_COLLECTION_HOST_V2 = (
    'application/vnd.org.midonet.collection.Host-v2+json',
)
CT_COLLECTION_TZONE_V1 = (
    'application/vnd.org.midonet.collection.TunnelZone-v1+json',
)
CT_COLLECTION_GRE_TZONE_HOST_V1 = (
    'application/vnd.org.midonet.collection.GreTunnelZoneHost-v1+json',
)
CT_COLLECTION_HOST_INTERFACE_PORT_V1 = (
    'application/vnd.org.midonet.collection.HostInterfacePort-v1+json',
)
CT_COLLECTION_PORT_GROUP_V1 = (
    'application/vnd.org.midonet.collection.PortGroup-v1+json',
)
CT_COLLECTION_PORT_GROUP_PORT_V1 = (
    'application/vnd.org.midonet.collection.PortGroupPort-v1+json',
)


def _uri(path, *args):
    return 'http://{}:{}/midonet-api{}'.format(
        API_HOST,
        API_PORT,
        path.format(*args),
    )


def _body_login(uuid_token):
    uuid_token = str(uuid_token.hex)

    return {
        'expires': 'Tue, 19-01-2038 03:14:07 GMT',
        'key': uuid_token,
    }


def _body_root():
    return {
        'adRouteTemplate': _uri('/ad_routes/{{id}}'),
        'bgpTemplate': _uri('/bgps/{{id}}'),
        'bridgeTemplate': _uri('/bridges/{{id}}'),
        'bridges': _uri('/bridges'),
        'chainTemplate': _uri('/chains/{{id}}'),
        'chains': _uri('/chains'),
        'healthMonitorTemplate': _uri('/health_monitors/{{id}}'),
        'healthMonitors': _uri('/health_monitors'),
        'hostTemplate': _uri('/hosts/{{id}}'),
        'hostVersions': _uri('/versions'),
        'hosts': _uri('/hosts'),
        'ipAddrGroupTemplate': _uri('/ip_addr_groups/{{id}}'),
        'ipAddrGroups': _uri('/ip_addr_groups'),
        'licenseStatus': None,
        'licenseTemplate': None,
        'licenses': None,
        'loadBalancerTemplate': _uri('/load_balancers/{{id}}'),
        'loadBalancers': _uri('/load_balancers'),
        'neutron': _uri('/neutron'),
        'poolMemberTemplate': _uri('/pool_members/{{id}}'),
        'poolMembers': _uri('/pool_members'),
        'poolTemplate': _uri('/pools/{{id}}'),
        'pools': _uri('/pools'),
        'portGroupTemplate': _uri('/port_groups/{{id}}'),
        'portGroups': _uri('/port_groups'),
        'portTemplate': _uri('/ports/{{id}}'),
        'ports': _uri('/ports'),
        'routeTemplate': _uri('/routes/{{id}}'),
        'routerTemplate': _uri('/routers/{{id}}'),
        'routers': _uri('/routers'),
        'ruleTemplate': _uri('/rules/{{id}}'),
        'systemState': _uri('/system_state'),
        'tenantTemplate': _uri('/tenants/{{id}}'),
        'tenants': _uri('/tenants'),
        'traceRequests': _uri('/traces'),
        'tunnelZoneTemplate': _uri('/tunnel_zones/{{id}}'),
        'tunnelZones': _uri('/tunnel_zones'),
        'uri': _uri('/'),
        'version': _uri('v1.9'),
        'vipTemplate': _uri('/vips/{{id}}'),
        'vips': _uri('/vips'),
        'vtepTemplate': _uri('/vteps/{{ipAddr}}'),
        'vteps': _uri('/vteps'),
        'writeVersion': _uri('/write_version'),
    }


def _body_system_state():
    return {
        'availability': 'READWRITE',
        'state': 'ACTIVE',
        'uri': _uri('/system_state'),
        'writeVersion': '1.9',
    }


def _body_router(uuid_id, name):
    uuid_id = str(uuid_id)
    name = str(name)

    return {
        'adminStateUp': True,
        'id': uuid_id,
        'inboundFilter': None,
        'inboundFilterId': None,
        'loadBalancer': None,
        'loadBalancerId': None,
        'name': name,
        'outboundFilter': None,
        'outboundFilterId': None,
        'peerPorts': _uri('/routers/{}/peer_ports', uuid_id),
        'ports': _uri('/routers/{}/ports', uuid_id),
        'routes': _uri('/routers/{}/routes', uuid_id),
        'tenantId': None,
        'uri': _uri('/routers/{}', uuid_id),
    }


def _body_port(uuid_id, uuid_host, uuid_router, address):
    uuid_id = str(uuid_id)
    uuid_host = str(uuid_host)
    uuid_router = str(uuid_router)
    address_ip = str(address.ip)
    address_network = str(address.network)
    address_prefixlen = int(address.prefixlen)

    return {
        'active': True,
        'adminStateUp': True,
        'bgps': _uri('/ports/{}/bgps', uuid_id),
        'device': _uri('/routers/{}', uuid_router),
        'deviceId': uuid_router,
        'host': _uri('/hosts/{}', uuid_host),
        'hostId': uuid_host,
        'hostInterfacePort': _uri('/hosts/{}/ports/{}', uuid_host, uuid_id),
        'id': uuid_id,
        'inboundFilter': None,
        'inboundFilterId': None,
        'interfaceName': None,
        'link': _uri('/ports/{}/link', uuid_id),
        'networkAddress': address_network,
        'networkLength': address_prefixlen,
        'outboundFilter': None,
        'outboundFilterId': None,
        'peer': None,
        'peerId': None,
        'portAddress': address_ip,
        'portGroups': _uri('/ports/{}/port_groups', uuid_id),
        'portMac': '00:53:00:00:00:01',
        'type': 'Router',
        'uri': _uri('/ports/{}', uuid_id),
        'vifId': None,
        'vlanId': -1,
    }


def _body_bgp(uuid_id, uuid_port, local_as, peer_address, peer_as):
    uuid_id = str(uuid_id)
    uuid_port = str(uuid_port)
    local_as = int(local_as)
    peer_address = str(peer_address)
    peer_as = int(peer_as)

    return {
        'adRoutes': _uri('/bgps/{}/ad_routes', uuid_id),
        'id': uuid_id,
        'localAS': local_as,
        'peerAS': peer_as,
        'peerAddr': peer_address,
        'port': _uri('/ports/{}', uuid_port),
        'portId': uuid_port,
        'status': '',
        'uri': _uri('/bgps/{}', uuid_id),
    }


def _body_route(uuid_id,  # pylint: disable=too-many-arguments
                uuid_router,
                src,
                dst,
                nexthop_uuid,
                next_hop_gateway):
    uuid_id = str(uuid_id)
    uuid_router = str(uuid_router)
    src_network = str(src.network)
    src_prefixlen = int(src.prefixlen)
    dst_network = str(dst.network)
    dst_prefixlen = int(dst.prefixlen)
    nexthop_uuid = str(nexthop_uuid)
    next_hop_gateway = str(next_hop_gateway)

    return {
        'attributes': None,
        'dstNetworkAddr': dst_network,
        'dstNetworkLength': dst_prefixlen,
        'id': uuid_id,
        'learned': False,
        'nextHopGateway': next_hop_gateway,
        'nextHopPort': nexthop_uuid,
        'router': _uri('/routers/{}', uuid_router),
        'routerId': uuid_router,
        'srcNetworkAddr': src_network,
        'srcNetworkLength': src_prefixlen,
        'type': 'Normal',
        'uri': _uri('/routes/{}', uuid_id),
        'weight': 0,
    }


def _body_host(uuid_id, hostname):
    uuid_id = str(uuid_id)
    hostname = str(hostname)

    return {
        'addresses': [],
        'alive': True,
        'floodingProxyWeight': 1,
        'id': uuid_id,
        'interfaces': _uri('/hosts/{}/interfaces', uuid_id),
        'name': hostname,
        'ports': _uri('/hosts/{}/ports', uuid_id),
        'uri': _uri('/hosts/{}', uuid_id),
    }


def _body_tunnel_zone(uuid_id, tzone_name, tzone_type):
    uuid_id = str(uuid_id)
    tzone_name = str(tzone_name)
    tzone_type = str(tzone_type)

    return{
        'hosts': _uri('/tunnel_zones/{}/hosts', uuid_id),
        'id': uuid_id,
        'name': tzone_name,
        'type': tzone_type,
        'uri': _uri('/tunnel_zones/{}', uuid_id),
    }


def _body_tunnel_zone_host(uuid_tzone, uuid_host, ipaddress):
    uuid_tzone = str(uuid_tzone)
    uuid_host = str(uuid_host)
    ipaddress = str(ipaddress)

    return {
        'hostId': uuid_host,
        'ipAddress': ipaddress,
        'tunnelZoneId': uuid_tzone,
        'uri': _uri('/tunnel_zones/{}/hosts/{}', uuid_tzone, uuid_host),
    }


def _body_host_port(uuid_host, uuid_port, interface):
    uuid_host = str(uuid_host)
    uuid_port = str(uuid_port)
    interface = str(interface)

    return {
        'host': _uri('/hosts/{}', uuid_host),
        'hostId': uuid_host,
        'interfaceName': interface,
        'port': _uri('/ports/{}', uuid_port),
        'portId': uuid_port,
        'uri': _uri('/hosts/{}/ports/{}', uuid_host, uuid_port),
    }


def _body_port_groups(uuid_id, name, tenant_id):
    uuid_id = str(uuid_id)
    name = str(name)
    tenant_id = str(tenant_id)

    return {
        'id': uuid_id,
        'name': name,
        'ports': _uri('/port_groups/{}/ports', uuid_id),
        'stateful': True,
        'tenantId': tenant_id,
        'uri': _uri('/port_groups/{}', uuid_id),
    }


def _body_port_group_port(uuid_port_group, uuid_port):
    uuid_port_group = str(uuid_port_group)
    uuid_port = str(uuid_port)

    return {
        'port': _uri('/ports/{}', uuid_port),
        'portGroup': _uri('/port_groups/{}', uuid_port_group),
        'portGroupId': uuid_port_group,
        'portId': uuid_port,
        'uri': _uri('/port_groups/{}/ports/{}', uuid_port_group, uuid_port),
    }


def get_bogus_midonetclient():
    return lib.misc.get_midonetclient(
        {
            'service_username': 'username',
            'service_password': 'password',
        },
        {
            'host': API_HOST,
            'port': API_PORT,
        },
    )


def mock_login(uuid_token):
    httpretty.register_uri(
        uri=_uri('/login'),
        method=httpretty.POST,
        status=200,
        set_cookie=(
            'sessionId={}; Expires=Tue, 19-01-2038 03:14:07 GMT'.format(
                uuid_token,
            ),
        ),
        content_type=CT_TOKEN_V1,
        body=json.dumps(_body_login(uuid_token)),
    )


def mock_root():
    httpretty.register_uri(
        uri=_uri(''),
        method=httpretty.GET,
        status=200,
        content_type=CT_APPLICATION_V5,
        body=json.dumps(_body_root()),
    )


def mock_system_state():
    httpretty.register_uri(
        uri=_uri('/system_state'),
        method=httpretty.GET,
        status=200,
        content_type=CT_SYSTEM_STATE_V2,
        body=json.dumps(_body_system_state()),
    )


def mock_routers_list(uuid_id, name):
    httpretty.register_uri(
        uri=_uri('/routers'),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_ROUTER_V2,
        body=json.dumps([_body_router(uuid_id, name)]),
    )


def mock_router_port_list(uuid_id, uuid_host, uuid_router, address):
    httpretty.register_uri(
        uri=_uri('/routers/{}/ports', uuid_router),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_V2,
        body=json.dumps([
            _body_port(uuid_id, uuid_host, uuid_router, address),
        ]),
    )


def mock_router_port_add(uuid_id, uuid_router):
    httpretty.register_uri(
        uri=_uri('/routers/{}/ports', uuid_router),
        method=httpretty.POST,
        status=201,
        location=_uri('/ports/{}', uuid_id),
    )


def mock_port_get(uuid_id, uuid_host, uuid_router, address):
    httpretty.register_uri(
        uri=_uri('/ports/{}', uuid_id),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_V2,
        body=json.dumps(_body_port(uuid_id, uuid_host, uuid_router, address)),
    )


def mock_port_delete(uuid_id):
    httpretty.register_uri(
        uri=_uri('/ports/{}', uuid_id),
        method=httpretty.DELETE,
        status=204,
    )


def mock_port_bgp_list(uuid_id, uuid_port, local_as, peer_address, peer_as):
    httpretty.register_uri(
        uri=_uri('/ports/{}/bgps', uuid_port),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_BGP_V1,
        body=json.dumps([
            _body_bgp(uuid_id, uuid_port, local_as, peer_address, peer_as),
        ]),
    )


def mock_port_bgp_add(uuid_id, uuid_port):
    httpretty.register_uri(
        uri=_uri('/ports/{}/bgps', uuid_port),
        method=httpretty.POST,
        status=201,
        location=_uri('/bgps/{}', uuid_id),
    )


def mock_bgp_get(uuid_id, uuid_port, local_as, peer_address, peer_as):
    httpretty.register_uri(
        uri=_uri('/bgps/{}', uuid_id),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_BGP_V1,
        body=json.dumps(
            _body_bgp(uuid_id, uuid_port, local_as, peer_address, peer_as),
        ),
    )


def mock_routers_route_list(uuid_id,  # pylint: disable=too-many-arguments
                            uuid_router,
                            src,
                            dst,
                            nexthop_uuid,
                            next_hop_gateway=None):
    httpretty.register_uri(
        uri=_uri('/routers/{}/routes', uuid_router),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_ROUTE_V1,
        body=json.dumps([
            _body_route(
                uuid_id,
                uuid_router,
                src,
                dst,
                nexthop_uuid,
                next_hop_gateway,
            ),
        ]),
    )


def mock_routers_route_add(uuid_id, uuid_router):
    httpretty.register_uri(
        uri=_uri('/routers/{}/routes', uuid_router),
        method=httpretty.POST,
        status=201,
        location=_uri('/routes/{}', uuid_id),
    )


def mock_route_delete(uuid_id):
    httpretty.register_uri(
        uri=_uri('/routes/{}', uuid_id),
        method=httpretty.DELETE,
        status=204,
    )


def mock_route_get(uuid_id,  # pylint: disable=too-many-arguments
                   uuid_router,
                   src,
                   dst,
                   nexthop_uuid,
                   next_hop_gateway=None):
    httpretty.register_uri(
        uri=_uri('/routes/{}', uuid_id),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_ROUTE_V1,
        body=json.dumps(
            _body_route(
                uuid_id,
                uuid_router,
                src,
                dst,
                nexthop_uuid,
                next_hop_gateway,
            ),
        ),
    )


def mock_host_list(uuid_id, hostname):
    httpretty.register_uri(
        uri=_uri('/hosts'),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_HOST_V2,
        body=json.dumps([_body_host(uuid_id, hostname)]),
    )


def mock_tunnel_zone_list(uuid_id, tzone_name, tzone_type):
    httpretty.register_uri(
        uri=_uri('/tunnel_zones'),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_TZONE_V1,
        body=json.dumps([_body_tunnel_zone(uuid_id, tzone_name, tzone_type)]),
    )


def mock_tunnel_zone_get(uuid_id, tzone_name, tzone_type):
    httpretty.register_uri(
        uri=_uri('/tunnel_zones/{}', uuid_id),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_TZONE_V1,
        body=json.dumps(_body_tunnel_zone(uuid_id, tzone_name, tzone_type)),
    )


def mock_tunnel_zone_host_list(uuid_tzone, uuid_host, ipaddress):
    httpretty.register_uri(
        uri=_uri('/tunnel_zones/{}/hosts', uuid_tzone),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_GRE_TZONE_HOST_V1,
        body=json.dumps([
            _body_tunnel_zone_host(uuid_tzone, uuid_host, ipaddress),
        ]),
    )


def mock_host_port_list(uuid_host, uuid_port, interface):
    httpretty.register_uri(
        uri=_uri('/hosts/{}/ports', uuid_host),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_HOST_INTERFACE_PORT_V1,
        body=json.dumps([_body_host_port(uuid_host, uuid_port, interface)]),
    )


def mock_host_port_add(uuid_host, uuid_port):
    httpretty.register_uri(
        uri=_uri('/hosts/{}/ports', uuid_host),
        method=httpretty.POST,
        status=201,
        location=_uri('/hosts/{}/ports/{}', uuid_host, uuid_port),
    )


def mock_host_port_get(uuid_host, uuid_port, interface):
    httpretty.register_uri(
        uri=_uri('/hosts/{}/ports/{}', uuid_host, uuid_port),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_HOST_INTERFACE_PORT_V1,
        body=json.dumps(_body_host_port(uuid_host, uuid_port, interface)),
    )


def mock_host_interface_port_delete(uuid_host, uuid_port):
    httpretty.register_uri(
        uri=_uri('/hosts/{}/ports/{}', uuid_host, uuid_port),
        method=httpretty.DELETE,
        status=204,
    )


def mock_port_group_list(uuid_id, name, tenant_id):
    httpretty.register_uri(
        uri=_uri('/port_groups'),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_GROUP_V1,
        body=json.dumps([_body_port_groups(uuid_id, name, tenant_id)]),
    )


def mock_port_group_add(uuid_id):
    httpretty.register_uri(
        uri=_uri('/port_groups'),
        method=httpretty.POST,
        status=201,
        location=_uri('/port_groups/{}', uuid_id),
    )


def mock_port_group_get(uuid_id, name, tenant_id):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}', uuid_id),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_GROUP_V1,
        body=json.dumps(_body_port_groups(uuid_id, name, tenant_id)),
    )


def mock_port_group_delete(uuid_id):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}', uuid_id),
        method=httpretty.DELETE,
        status=204,
    )


def mock_port_group_port_list(uuid_port_group, uuid_port):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}/ports', uuid_port_group),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_GROUP_PORT_V1,
        body=json.dumps([_body_port_group_port(uuid_port_group, uuid_port)]),
    )


def mock_port_group_port_list_empty(uuid_port_group):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}/ports', uuid_port_group),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_GROUP_PORT_V1,
        body=json.dumps([]),
    )


def mock_port_group_port_add(uuid_port_group, uuid_port):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}/ports', uuid_port_group),
        method=httpretty.POST,
        status=201,
        location=_uri('/port_groups/{}/ports/{}', uuid_port_group, uuid_port),
    )


def mock_port_group_port_get(uuid_port_group, uuid_port):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}/ports/{}', uuid_port_group, uuid_port),
        method=httpretty.GET,
        status=200,
        content_type=CT_COLLECTION_PORT_GROUP_PORT_V1,
        body=json.dumps(_body_port_group_port(uuid_port_group, uuid_port)),
    )


def mock_port_group_port_delete(uuid_port_group, uuid_port):
    httpretty.register_uri(
        uri=_uri('/port_groups/{}/ports/{}', uuid_port_group, uuid_port),
        method=httpretty.DELETE,
        status=204,
    )
