from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os

import mock
import mock_open
import nose.tools

import lib.config
import lib.tests.mock_config
import lib.tests.mock_midonet_api
import lib.tests.mock_neutron_api


class TestMidonetGatewayRequiredConfig(object):
    def setup(self):
        charm_dir = os.path.abspath(
            os.path.join(
                __file__,
                os.path.pardir,
                os.path.pardir,
                os.path.pardir,
            ),
        )
        config_yaml = os.path.join(charm_dir, 'config.yaml')

        with open(config_yaml) as file_object:
            config_yaml_content = file_object.read()

        self.patcher_local_config = mock.patch(
            'charmhelpers.core.hookenv.config',
        )
        self.mocked_local_config = self.patcher_local_config.start()

        self.patcher_local_charm_dir = mock.patch(
            'charmhelpers.core.hookenv.charm_dir',
        )
        self.mocked_local_charm_dir = self.patcher_local_charm_dir.start()
        self.mocked_local_charm_dir.return_value = charm_dir

        self.patcher_open = mock.patch(
            '__builtin__.open',
            mock_open.MockOpen(),
        )
        self.mocked_open = self.patcher_open.start()
        self.mocked_open[config_yaml].read_data = config_yaml_content

    def teardown(self):
        self.patcher_local_config.stop()
        self.patcher_local_charm_dir.stop()
        self.patcher_open.stop()

    def test_static(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.READY_STATIC
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_true(bool(midonet_gateway_required_config))

    def test_empty(self):
        self.mocked_local_config.return_value = ({})

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_value(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_midonet_flavor(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_MISSING_MIDONET_FLAVOR
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_wrong_midonet_flavor(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_WRONG_MIDONET_FLAVOR
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_uplink_type(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_MISSING_UPLINK_TYPE
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_wrong_uplink_type(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_WRONG_UPLINK_TYPE
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_enterprise_blank_username(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_ENTERPRISE_NO_USERNAME
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_enterprise_blank_password(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_ENTERPRISE_NO_PASSWORD
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_static_key(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_MISSING_STATIC_KEY
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_static_value(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_MISSING_STATIC_VALUE
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_static_masq_key(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_MISSING_STATIC_MASQ_KEY
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))

    def test_missing_static_masq_value(self):
        self.mocked_local_config.return_value = (
            lib.tests.mock_config.NOT_READY_MISSING_STATIC_MASQ_VALUE
        )

        midonet_gateway_required_config = (
            lib.config.MidonetGatewayRequiredConfig()
        )

        nose.tools.assert_false(bool(midonet_gateway_required_config))
