from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import uuid

import charmhelpers.core.hookenv
import httpretty
import mock
import mock_open
import netaddr
import nose.tools

import lib.midonet_conf
import lib.tests.mock_midonet_conf
import lib.tests.mocks.midonet


UUID_AUTH_TOKEN = uuid.uuid4()
UUID_ROUTER = uuid.uuid4()
UUID_ROUTE = uuid.uuid4()
UUID_PORT = uuid.uuid4()
UUID_HOST = uuid.uuid4()
UUID_PORT_GROUP = uuid.uuid4()
UUID_TENANT_ID = uuid.uuid4()


class TestSetupStaticMidonetGateway(object):
    def setup(self):
        self.rd_config = {
            'static-linux-bridge-address': (
                lib.tests.mock_midonet_conf.STATIC_LINUX_BRIDGE_ADDRESS
            ),
            'static-provider-router-address': (
                lib.tests.mock_midonet_conf.STATIC_PROVIDER_ROUTER_ADDRESS
            ),
            'static-forwarding-network': (
                lib.tests.mock_midonet_conf.STATIC_FORWARDING_NETWORK
            ),
        }

        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        self.patcher_open = mock.patch(
            '__builtin__.open',
            mock_open.MockOpen(),
        )
        self.mocked_open = self.patcher_open.start()
        self.mocked_open['/etc/midonet_host_id.properties'].read_data = str(
            '#Tue Jan 19 03:14:07 UTC 2015\n'
            'host_uuid={}'.format(
                lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
            ),
        )

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_conf.mock_login()
        lib.tests.mock_midonet_conf.mock_root()
        lib.tests.mock_midonet_conf.mock_system_state()
        lib.tests.mock_midonet_conf.mock_routers_list_pass()

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_open.stop()
        self.patcher_log.stop()

    def test_first_pass(self):
        lib.tests.mock_midonet_conf.mock_router_port_list_fail()
        lib.tests.mock_midonet_conf.mock_router_port_add()
        lib.tests.mock_midonet_conf.mock_port_get()
        lib.tests.mock_midonet_conf.mock_routers_route_list_fail()
        lib.tests.mock_midonet_conf.mock_routers_route_add()
        lib.tests.mock_midonet_conf.mock_route_get()
        lib.tests.mock_midonet_conf.mock_host_list()
        lib.tests.mock_midonet_conf.mock_tunnel_zone_list()
        lib.tests.mock_midonet_conf.mock_tunnel_zone_get()
        lib.tests.mock_midonet_conf.mock_tunnel_zone_host_list()
        lib.tests.mock_midonet_conf.mock_host_port_list_fail()
        lib.tests.mock_midonet_conf.mock_host_port_add()
        lib.tests.mock_midonet_conf.mock_host_port_get()

        lib.midonet_conf.setup_static_midonet_gateway(
            lib.tests.mocks.midonet.get_bogus_midonetclient(),
            self.rd_config,
        )

        provider_router_address = netaddr.IPNetwork(
            lib.tests.mock_midonet_conf.STATIC_PROVIDER_ROUTER_ADDRESS,
        )

        linux_bridge_address = netaddr.IPNetwork(
            lib.tests.mock_midonet_conf.STATIC_LINUX_BRIDGE_ADDRESS
        ).ip

        asserted_calls = [
            mock.call(
                lib.log.MIDONET_SETUP_STARTED_STATIC,
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_PROVIDER_ROUTER_FOUND.format(
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTER_PORT_NOT_FOUND.format(
                    str(provider_router_address.ip),
                    str(provider_router_address.prefixlen),
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTER_PORT_CREATED.format(
                    str(provider_router_address.ip),
                    str(provider_router_address.prefixlen),
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTER_ROUTE_NOT_FOUND.format(
                    '0.0.0.0/0',
                    '0.0.0.0/0',
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    linux_bridge_address,
                    'Normal',
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTER_ROUTE_CREATED.format(
                    '0.0.0.0/0',
                    '0.0.0.0/0',
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    linux_bridge_address,
                    'Normal',
                    lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_FOUND.format(
                    lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                    lib.tests.mock_midonet_conf.SOME_HOST_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_IN_A_TZONE.format(
                    lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                    lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_INTERFACE_PORT_NOT_FOUND.format(
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_INTERFACE_PORT_CREATED.format(
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                    lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        ]
        nose.tools.assert_equal(self.mocked_log.mock_calls, asserted_calls)

    def test_second_pass(self):
        lib.tests.mock_midonet_conf.mock_router_port_list_pass()
        lib.tests.mock_midonet_conf.mock_routers_route_list_pass()
        lib.tests.mock_midonet_conf.mock_host_list()
        lib.tests.mock_midonet_conf.mock_tunnel_zone_list()
        lib.tests.mock_midonet_conf.mock_tunnel_zone_get()
        lib.tests.mock_midonet_conf.mock_tunnel_zone_host_list()
        lib.tests.mock_midonet_conf.mock_host_port_list_pass()

        lib.midonet_conf.setup_static_midonet_gateway(
            lib.tests.mocks.midonet.get_bogus_midonetclient(),
            self.rd_config,
        )

        provider_router_address = netaddr.IPNetwork(
            lib.tests.mock_midonet_conf.STATIC_PROVIDER_ROUTER_ADDRESS,
        )

        linux_bridge_address = netaddr.IPNetwork(
            lib.tests.mock_midonet_conf.STATIC_LINUX_BRIDGE_ADDRESS
        ).ip

        asserted_calls = [
            mock.call(
                lib.log.MIDONET_SETUP_STARTED_STATIC,
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_PROVIDER_ROUTER_FOUND.format(
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTER_PORT_FOUND.format(
                    str(provider_router_address.ip),
                    str(provider_router_address.prefixlen),
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTER_ROUTE_FOUND.format(
                    '0.0.0.0/0',
                    '0.0.0.0/0',
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    linux_bridge_address,
                    'Normal',
                    lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_FOUND.format(
                    lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                    lib.tests.mock_midonet_conf.SOME_HOST_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_IN_A_TZONE.format(
                    lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                    lib.tests.mocks.midonet.UUID_TZONE_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_HOST_INTERFACE_PORT_FOUND.format(
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                    lib.tests.mocks.midonet.UUID_HOST_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        ]
        nose.tools.assert_equal(self.mocked_log.mock_calls, asserted_calls)
