from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock
import nose.tools

import lib.neutron_api
import lib.tests.mock_neutron_api


class TestNetwork(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        self.patcher_wait_midonet_api = mock.patch(
            'lib.workarounds.wait_for_midonet_api_to_settle',
        )
        self.mocked_wait_midonet_api = self.patcher_wait_midonet_api.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_neutron_api.mock_keystone_tokens()

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_wait_midonet_api.stop()
        self.patcher_log.stop()

    def test_get_pass(self):
        lib.tests.mock_neutron_api.mock_net_list_pass()

        external_network = lib.neutron_api.get_external_network(
            lib.tests.mock_neutron_api.get_bogus_neutronclient(),
            lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
        )

        nose.tools.assert_equal(
            external_network['id'],
            str(lib.tests.mock_neutron_api.UUID_NEUTRON_NET_EXTERNAL_PASS)
        )
        nose.tools.assert_equal(
            external_network['name'],
            str(lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS)
        )
        self.mocked_log.assert_called_once_with(
            lib.log.NEUTRON_EXTERNAL_NETWORK_FOUND.format(
                lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
                lib.tests.mock_neutron_api.UUID_NEUTRON_NET_EXTERNAL_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_get_external_network_fail(self):
        lib.tests.mock_neutron_api.mock_net_list_fail()

        external_network = lib.neutron_api.get_external_network(
            lib.tests.mock_neutron_api.get_bogus_neutronclient(),
            lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
        )

        nose.tools.assert_is_none(
            external_network,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.NEUTRON_EXTERNAL_NETWORK_NOT_FOUND.format(
                lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
                lib.tests.mock_neutron_api.UUID_NEUTRON_NET_EXTERNAL_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_create_external_network(self):
        lib.tests.mock_neutron_api.mock_net_list_fail()
        lib.tests.mock_neutron_api.mock_net_create()

        external_network = lib.neutron_api.create_external_network(
            lib.tests.mock_neutron_api.get_bogus_neutronclient(),
            lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
        )

        nose.tools.assert_equal(
            external_network['id'],
            str(lib.tests.mock_neutron_api.UUID_NEUTRON_NET_EXTERNAL_PASS)
        )
        nose.tools.assert_equal(
            external_network['name'],
            str(lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS)
        )

        asserted_calls = [
            mock.call(
                lib.log.NEUTRON_EXTERNAL_NETWORK_NOT_FOUND.format(
                    lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
                    lib.tests.mock_neutron_api.UUID_NEUTRON_NET_EXTERNAL_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.NEUTRON_EXTERNAL_NETWORK_CREATED.format(
                    lib.tests.mock_neutron_api.EXTERNAL_NETWORK_NAME_PASS,
                    lib.tests.mock_neutron_api.UUID_NEUTRON_NET_EXTERNAL_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        ]
        nose.tools.assert_equal(self.mocked_log.mock_calls, asserted_calls)
