from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import subprocess
import time

import requests
import retrying


def wait_for_midonet_api_to_settle():
    subprocess.check_call([
        'juju-log', '--debug',
        '*** WORKAROUND | Waiting for Tomcat to settle | WORKAROUND ***',
    ])

    relation_id = json.loads(subprocess.check_output(
        ['relation-ids', '--format=json', 'midonet-api']
    ))[0]
    unit_id = json.loads(subprocess.check_output(
        ['relation-list', '--format=json', '-r', relation_id]
    ))[0]
    relation_data = json.loads(subprocess.check_output(
        ['relation-get', '--format=json', '-r', relation_id, '-', unit_id]
    ))

    midonet_api_host = str(relation_data['private-address'])
    midonet_api_port = int(relation_data['port'])

    _wait_for_midonet_api_to_settle(midonet_api_host, midonet_api_port)

    subprocess.check_call([
        'juju-log', '--debug',
        '*** WORKAROUND | Tomcat seems to be settled | WORKAROUND ***',
    ])


def _retry_if_midonet_api_error(exception):
    conditions = []
    conditions.append(isinstance(
        exception,
        requests.exceptions.RequestException,
    ))

    return any(conditions)


@retrying.retry(stop_max_delay=(1800*1000), wait_fixed=(3*1000),
                retry_on_exception=_retry_if_midonet_api_error)
def _wait_for_midonet_api_to_settle(midonet_api_host, midonet_api_port):
    for i in xrange(60):
        subprocess.check_call([
            'juju-log', '--debug',
            '*** WORKAROUND | Trying to establish a connection to Tomcat ({}) '
            '| WORKAROUND ***'.format(i),
        ])

        request = requests.get(
            'http://{}:{}/midonet-api'.format(
                midonet_api_host,
                midonet_api_port,
            ),
            timeout=3,
        )

        if request.status_code != 401:
            request.raise_for_status()

        subprocess.check_call([
            'juju-log', '--debug',
            '*** WORKAROUND | Successfully established a connection to Tomcat,'
            ' sleeping for 1 second | WORKAROUND ***',
        ])

        time.sleep(1)


if __name__ == '__main__':
    wait_for_midonet_api_to_settle()
